'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _stream = require('stream');

var _nodeObjectHash = require('node-object-hash');

var _nodeObjectHash2 = _interopRequireDefault(_nodeObjectHash);

var _constants = require('./constants');

var _SafeDuplex2 = require('./SafeDuplex');

var _SafeDuplex3 = _interopRequireDefault(_SafeDuplex2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var hashCoerce = (0, _nodeObjectHash2.default)({
    sort: false,
    coerce: true
});

var Booster = function (_SafeDuplex) {
    _inherits(Booster, _SafeDuplex);

    function Booster(ezs, commands, environment) {
        _classCallCheck(this, Booster);

        var _this = _possibleConstructorReturn(this, (Booster.__proto__ || Object.getPrototypeOf(Booster)).call(this, ezs.objectMode()));

        _this.ezs = ezs;
        _this.commands = commands;
        _this.environment = environment;
        _this.initialized = false;
        return _this;
    }

    _createClass(Booster, [{
        key: 'initialize',
        value: function initialize() {
            // to avoid to create stream even if booster is not used
            if (!this.initialized) {
                var ezs = this.ezs;

                this.commandsHash = hashCoerce.hash(this.commands);
                this.environmentHash = hashCoerce.hash(this.environment);
                this.pipeline = ezs('delegate', { commands: this.commands }, this.environment);
                this.initialized = true;
                this.firstWrite = true;
                this.readCalled = false;
                this.isCached = false;
                this.cacheHandle = null;
                this.uniqHash = null;
                this.cacheInput = new _stream.PassThrough(ezs.objectMode());
                this.cacheOutput = this.cacheInput.pipe(ezs('group')).pipe(ezs('pack')).pipe(ezs.compress(ezs.encodingMode()));
            }
        }
    }, {
        key: '_write',
        value: function _write(chunk, encoding, callback) {
            var _this2 = this;

            this.initialize();
            var ezs = this.ezs;

            if (this.firstWrite) {
                this.firstWrite = false;
                var ignoreChunk = true;
                var firstChunkHash = hashCoerce.hash(chunk);
                var hashs = [this.commandsHash, this.environmentHash, firstChunkHash];
                (0, _constants.DEBUG)('Compute cache hash with', hashs.map(function (h) {
                    return h.slice(0, 5).concat('...');
                }));
                this.uniqHash = hashCoerce.hash(hashs);
                return ezs.getCache().has(this.uniqHash).then(function (cached) {
                    return new Promise(function (resolve) {
                        _this2.isCached = cached;
                        if (cached) {
                            (0, _constants.DEBUG)('Using cache with hash', _this2.uniqHash);
                            _this2.emit('cache:connected', _this2.uniqHash);
                            return ezs.getCache().get(_this2.uniqHash).catch(function (err) {
                                return _this2.failWith(err, 'get');
                            }).then(function (stream) {
                                return resolve(stream);
                            });
                        }
                        (0, _constants.DEBUG)('Creating cache with hash', _this2.uniqHash);
                        _this2.emit('cache:created', _this2.uniqHash);
                        _this2.cacheHandle = ezs.getCache().set(_this2.uniqHash, _this2.cacheOutput);
                        return resolve();
                    });
                }).then(function (stream) {
                    return new Promise(function (resolve) {
                        if (stream) {
                            var stream2 = stream.pipe(ezs.uncompress(ezs.encodingMode()));
                            stream2.on('error', function (err) {
                                return _this2.failWith(err, 'uncompress');
                            });

                            return resolve(stream2.pipe(ezs('unpack')).pipe(ezs('ungroup')).pipe(ezs(function (data, feed) {
                                if (data !== null) {
                                    if (!_this2.push(data)) {
                                        stream.pause();
                                    }
                                } else {
                                    _this2.push(null);
                                }
                                feed.send(data);
                            })).pipe(ezs.catch(function (err) {
                                return _this2.failWith(err, 'set(catch)');
                            })).on('error', function (err) {
                                return _this2.failWith(err, 'set(event)');
                            }));
                        }
                        ignoreChunk = false;
                        return resolve(_this2.pipeline.pipe(ezs(function (data, feed) {
                            if (data !== null) {
                                _this2.cacheInput.write(data, function () {
                                    return feed.send(data);
                                });
                            } else {
                                _this2.cacheInput.end(function () {
                                    return feed.send(data);
                                });
                            }
                        })).pipe(ezs(function (data, feed) {
                            if (data !== null) {
                                if (!_this2.push(data)) {
                                    stream.pause();
                                }
                            } else {
                                _this2.push(null);
                            }
                            feed.send(data);
                        })));
                    });
                }).then(function (output) {
                    _this2.output = output;
                    if (!_this2.readCalled) {
                        _this2.output.pause();
                    }
                    if (!ignoreChunk) {
                        _this2.pipeline.write(chunk, encoding);
                    }
                    return callback();
                }).catch(function (err) {
                    return _this2.failWith(err, 'output');
                });
            }
            if (this.isCached) {
                return callback();
            }
            return this.pipeline.write(chunk, encoding, callback);
        }
    }, {
        key: '_read',
        value: function _read() {
            this.initialize();
            if (!this.readCalled) {
                this.readCalled = true;
            }
            if (this.output) {
                this.output.resume();
            }
        }
    }, {
        key: '_final',
        value: function _final(callback) {
            var _this3 = this;

            this.pipeline.end(function () {
                if (_this3.cacheHandle) {
                    return _this3.cacheHandle.catch(function (err) {
                        return _this3.failWith(err);
                    }).then(function () {
                        return callback();
                    });
                }
                return callback();
            });
        }
    }, {
        key: '_destroy',
        value: function _destroy(err, callback) {
            this.pipeline.destroy();
            return callback();
        }
    }, {
        key: 'failWith',
        value: function failWith(err, typ) {
            var _this4 = this;

            // https://github.com/nodejs/node/issues/9242
            (0, _constants.DEBUG)('Booster failed with ' + err.message + ' - ' + typ);
            this.emit('error', err);
            if (this.isCached) {
                this.ezs.getCache().del(this.uniqHash).catch(function (e) {
                    return _this4.emit(e);
                });
            }
        }
    }]);

    return Booster;
}(_SafeDuplex3.default);

var booster = function booster(ezs, commands, environment) {
    return new Booster(ezs, commands, environment);
};

exports.default = booster;