'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _stream = require('stream');

var _engine = require('./engine');

var _engine2 = _interopRequireDefault(_engine);

var _script = require('./script');

var _script2 = _interopRequireDefault(_script);

var _file = require('./file');

var _file2 = _interopRequireDefault(_file);

var _output = require('./output');

var _output2 = _interopRequireDefault(_output);

var _commands = require('./commands');

var _commands2 = _interopRequireDefault(_commands);

var _catcher = require('./catcher');

var _catcher2 = _interopRequireDefault(_catcher);

var _statements = require('./statements');

var _statements2 = _interopRequireDefault(_statements);

var _statement = require('./statement');

var _statement2 = _interopRequireDefault(_statement);

var _meta = require('./meta');

var _meta2 = _interopRequireDefault(_meta);

var _server = require('./server');

var _server2 = _interopRequireDefault(_server);

var _settings = require('./settings');

var _settings2 = _interopRequireDefault(_settings);

var _compactor = require('./compactor');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var ezs = function ezs(name, options, environment) {
    return new _engine2.default(ezs, _statement2.default.get(ezs, name, options), options, environment);
};
var ezsPath = [process.cwd()];

ezs.settings = _settings2.default;
ezs.objectMode = function () {
    return {
        objectMode: true,
        highWaterMark: _settings2.default.highWaterMark.object
    };
};
ezs.bytesMode = function () {
    return {
        objectMode: false,
        highWaterMark: ezs.settings.highWaterMark.bytes
    };
};
ezs.encodingMode = function () {
    return {
        'Content-Encoding': _settings2.default.encoding
    };
};
ezs.fileToServe = function (file) {
    return _path2.default.join(_settings2.default.servePath, file);
};
ezs.metaString = function (commands, options) {
    return new _meta2.default(ezs, commands, options);
};
ezs.metaFile = function (filename, options) {
    return new _meta2.default(ezs, (0, _file2.default)(ezs, filename), options);
};
ezs.parseString = function (commands) {
    return (0, _script2.default)(commands);
};
ezs.parseFile = function (filename) {
    return (0, _script2.default)((0, _file2.default)(ezs, filename));
};
ezs.catch = function (func) {
    return new _catcher2.default(func);
};
ezs.toBuffer = function (options) {
    return new _output2.default(options);
};
ezs.use = function (plugin) {
    return _statement2.default.set(ezs, plugin);
};
ezs.addPath = function (p) {
    return ezsPath.push(p);
};
ezs.getPath = function () {
    return ezsPath;
};
ezs.loadScript = function (file) {
    return (0, _file2.default)(ezs, file);
};
ezs.compileScript = function (script) {
    return new _commands2.default(ezs.parseString(script));
};
ezs.createCommand = function (command, environment) {
    var mode = command.mode || _constants.M_NORMAL;
    if (!command.name) {
        throw new Error('Bad command : ' + command.name);
    }
    if (mode === _constants.M_NORMAL || mode === _constants.M_DISPATCH) {
        return ezs(command.name, command.args, environment);
    }
    if (mode === _constants.M_SINGLE || mode === 'single' /* Backward compatibility */) {
            return ezs('singleton', _extends({}, command.args, { statement: command.name }), environment);
        }
    throw new Error('Bad mode: ' + mode);
};
ezs.compileCommands = function (commands, environment) {
    if (!Array.isArray(commands)) {
        throw new Error('Pipeline works with an array of commands.');
    }
    var cmds = [].concat(_toConsumableArray(commands));
    cmds.push({
        mode: _constants.M_NORMAL,
        name: 'transit',
        args: {}
    });
    var streams = cmds.map(function (command) {
        return ezs.createCommand(command, environment);
    });
    if (streams.length === 1) {
        return new _stream.PassThrough(ezs.objectMode());
    }
    return streams;
};
ezs.writeTo = function (stream, data, callback) {
    var check = stream.write(data);
    if (!check) {
        stream.once('drain', callback);
    } else {
        process.nextTick(callback);
    }
    return check;
};
ezs.createPipeline = function (input, streams) {
    return streams.reduce(function (amont, aval) {
        return amont.pipe(aval);
    }, input);
};
ezs.compress = function (options) {
    return (0, _compactor.compressStream)(ezs, options);
};
ezs.uncompress = function (options) {
    return (0, _compactor.uncompressStream)(ezs, options);
};
ezs.createStream = function (options) {
    return new _stream.PassThrough(options);
};
ezs.createServer = function (port) {
    return _server2.default.createServer(ezs, port);
};
ezs.createCluster = function (port) {
    return _server2.default.createCluster(ezs, port);
};

ezs.use(_statements2.default);

module.exports = ezs;