'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _parameter = require('../parameter');

var _parameter2 = _interopRequireDefault(_parameter);

var _file = require('../file');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var knownPipeline = function knownPipeline(ezs) {
    return function (request, response) {
        var _request$url = request.url,
            pathname = _request$url.pathname,
            query = _request$url.query;

        var errorHandler = function errorHandler(error) {
            (0, _debug2.default)('ezs')('Server has caught an error', error);
            if (!response.headersSent) {
                response.writeHead(500, { 'X-Error': _parameter2.default.encode(error.toString()) });
                response.end();
            }
        };
        var files = pathname.slice(1).split(';').map(function (file) {
            return ezs.fileToServe(file);
        }).filter(function (file) {
            return (0, _file.isFile)(file);
        }).map(function (file) {
            return ezs('delegate', { file: file }, query);
        });
        if (files.length === 0) {
            response.writeHead(404);
            response.end();
            return false;
        }
        var headers = request.headers;

        response.setHeader('Content-Encoding', headers['content-encoding'] || 'identity');
        (0, _debug2.default)('ezs')('PID ' + process.pid + ' will execute ' + pathname + ' commands with ' + (Object.keys(query).length || 0) + ' global parameters');
        var input = request.pipe(ezs.uncompress(headers)).on('error', errorHandler);
        ezs.createPipeline(input, files).pipe(ezs.catch(function (e) {
            return e;
        })).on('error', errorHandler).pipe(ezs(function (data, feed) {
            if (!response.headersSent) {
                response.writeHead(200);
            }
            return feed.send(data);
        })).pipe(ezs.toBuffer()).pipe(ezs.compress(headers)).pipe(response);
        request.resume();
        return true;
    };
};

exports.default = knownPipeline;