'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = delegate;

var _stream = require('stream');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Takes an `Object` delegate processing to an external pipeline
 *
 * @param {String} [file] the external pipeline is descrbied in a file
 * @param {String} [script] the external pipeline is descrbied in a sting of characters
 * @param {String} [commands] the external pipeline is descrbied in object
 * @returns {Object}
 */
function delegate(data, feed) {
    var ezs = this.ezs;

    if (this.isFirst()) {
        var file = this.getParam('file');
        var fileContent = ezs.loadScript(file);
        var script = this.getParam('script', fileContent);
        var cmds = ezs.compileScript(script);
        var commands = this.getParam('commands', cmds.get());
        var environment = this.getEnv();

        if (!commands || commands.length === 0) {
            return feed.stop(new Error('Invalid parmeter for delegate'));
        }

        var streams = ezs.compileCommands(commands, environment);
        this.input = new _stream.PassThrough({ objectMode: true });

        var output = ezs.createPipeline(this.input, streams).pipe(ezs.catch(function (e) {
            return feed.write(e);
        })).on('error', function (e) {
            return feed.write(e);
        }).on('data', function (d) {
            return feed.write(d);
        });

        this.whenFinish = new Promise(function (resolve) {
            output.on('end', resolve);
        });
        (0, _debug2.default)('ezs')('Delegate first chunk #' + this.getIndex() + ' containing ' + (Object.keys(data).length || 0) + ' keys');
        return ezs.writeTo(this.input, data, function () {
            return feed.end();
        });
    }
    if (this.isLast()) {
        this.whenFinish.then(function () {
            return feed.close();
        }).catch(function (e) {
            return feed.stop(e);
        });
        return this.input.end();
    }
    (0, _debug2.default)('ezs')('Delegate chunk #' + this.getIndex() + ' containing ' + (Object.keys(data).length || 0) + ' keys');
    return ezs.writeTo(this.input, data, function () {
        return feed.end();
    });
}