'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = dispatch;

var _merge = require('merge2');

var _merge2 = _interopRequireDefault(_merge);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _client = require('../client');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Takes an `Object` dispatch processing to an external pipeline in one or more servers
 *
 * @param {String} [server] servers to dispatch data
 * @param {String} [file] the external pipeline is descrbied in a file
 * @param {String} [script] the external pipeline is descrbied in a sting of characters
 * @param {String} [commands] the external pipeline is descrbied in object
 * @returns {Object}
 */
function dispatch(data, feed) {
    var ezs = this.ezs;

    if (this.isFirst()) {
        this.lastIndex = 0;
        var file = this.getParam('file');
        var fileContent = ezs.loadScript(file);
        var script = this.getParam('script', fileContent);
        var cmds = ezs.compileScript(script);
        var commands = this.getParam('commands', cmds.get());
        var environment = this.getEnv();
        var servers = (0, _client.inspectServers)(this.getParam('server', []), commands, environment);

        if (!servers || servers.length === 0 || !commands || commands.length === 0) {
            return feed.stop(new Error('Invalid parmeter for dispatch'));
        }
        var handles = servers.map((0, _client.connectServer)(ezs));
        this.ins = handles.map(function (h) {
            return h[0];
        });
        this.outs = handles.map(function (h) {
            return h[1];
        });
        var funnel = (0, _merge2.default)(this.outs, ezs.objectMode()).on('queueDrain', function () {
            funnel.destroy();
        }).on('error', function (e) {
            return feed.write(e);
        }).on('data', function (d) {
            return feed.write(d);
        });
        this.whenFinish = new Promise(function (resolve) {
            funnel.on('close', resolve);
        });
    }
    if (this.isLast()) {
        this.whenFinish.then(function () {
            return feed.close();
        }).catch(function (e) {
            return feed.stop(e);
        });
        this.ins.forEach(function (handle) {
            return handle.end();
        });
    } else {
        if (this.lastIndex >= this.ins.length) {
            this.lastIndex = 0;
        }
        (0, _debug2.default)('ezs')('Write chunk #' + this.getIndex() + ' containing ' + (Object.keys(data).length || 0) + ' keys into handle #' + (this.lastIndex + 1) + '/' + this.ins.length);
        var check = ezs.writeTo(this.ins[this.lastIndex], data, function () {
            return feed.end();
        });
        if (!check) {
            this.lastIndex += 1;
        }
    }
    return 1;
}